@@ -14,6 +14,7 @@ class CalibrationModule(mp_module.MPModule):
         self.add_command('compassmot', self.cmd_compassmot, 'do compass/motor interference calibration')
         self.add_command('calpress', self.cmd_calpressure,'calibrate pressure sensors')
         self.add_command('accelcal', self.cmd_accelcal, 'do 3D accelerometer calibration')
+        self.add_command('accelcalsimple', self.cmd_accelcal_simple, 'do simple accelerometer calibration')
         self.add_command('gyrocal', self.cmd_gyrocal, 'do gyro calibration')
         self.add_command('ahrstrim', self.cmd_ahrstrim, 'do AHRS trim')
         self.add_command('magcal', self.cmd_magcal, "magcal")
@@ -29,7 +30,7 @@ class CalibrationModule(mp_module.MPModule):
 
     def cmd_level(self, args):
         '''run a accel level'''
-        self.master.calibrate_level()
+        print("level is no longer supported; use ahrstrim, accelcal or accelcalsimple")
 
     def cmd_accelcal(self, args):
         '''do a full 3D accel calibration'''
@@ -41,6 +42,13 @@ class CalibrationModule(mp_module.MPModule):
         self.accelcal_count = 0
         self.accelcal_wait_enter = False
 
+    def cmd_accelcal_simple(self, args):
+        '''do a simple accel calibration'''
+        mav = self.master
+        mav.mav.command_long_send(mav.target_system, mav.target_component,
+                                  mavutil.mavlink.MAV_CMD_PREFLIGHT_CALIBRATION, 0,
+                                  0, 0, 0, 0, 4, 0, 0)
+        
     def cmd_gyrocal(self, args):
         '''do a full gyro calibration'''
         mav = self.master
@@ -57,19 +65,20 @@ class CalibrationModule(mp_module.MPModule):
 
     def mavlink_packet(self, m):
         '''handle mavlink packets'''
+        mtype = m.get_type()
         if self.accelcal_count != -1:
-            if m.get_type() == 'STATUSTEXT':
+            if mtype == 'STATUSTEXT':
                 # handle accelcal packet
                 text = str(m.text)
                 if text.startswith('Place '):
                     self.accelcal_wait_enter = True
                     self.empty_input_count = self.mpstate.empty_input_count
-        if m.get_type() == 'MAG_CAL_PROGRESS':
+        if mtype == 'MAG_CAL_PROGRESS':
             while m.compass_id >= len(self.magcal_progess):
                 self.magcal_progess.append("")
             self.magcal_progess[m.compass_id] = "%u%%" % m.completion_pct
             self.console.set_status('Progress', 'Calibration Progress: ' + " ".join(self.magcal_progess), row=4)
-        if m.get_type() == 'MAG_CAL_REPORT':
+        elif mtype == 'MAG_CAL_REPORT':
             if m.cal_status == mavutil.mavlink.MAG_CAL_SUCCESS:
                 result = "SUCCESS"
             else:
@@ -77,6 +86,10 @@ class CalibrationModule(mp_module.MPModule):
             self.magcal_progess[m.compass_id] = result
             self.console.set_status('Progress', 'Calibration Progress: ' + " ".join(self.magcal_progess), row=4)
             print("Calibration of compass %u %s: fitness %.3f" % (m.compass_id, result, m.fitness))
+            mav = self.master
+            mav.mav.command_long_send(mav.target_system, mav.target_component,
+                                      mavutil.mavlink.MAV_CMD_DO_ACCEPT_MAG_CAL, 0,
+                                      1<<m.compass_id, 0, 0, 0, 0, 0, 0)
 
     def idle_task(self):
         '''handle mavlink packets'''
@@ -96,7 +109,7 @@ class CalibrationModule(mp_module.MPModule):
                     print("sending stop")
                     self.master.mav.command_ack_send(0, 1)
 
-    
+
     def cmd_compassmot(self, args):
         '''do a compass/motor interference calibration'''
         mav = self.master
@@ -111,10 +124,13 @@ class CalibrationModule(mp_module.MPModule):
         '''calibrate pressure sensors'''
         self.master.calibrate_pressure()
 
+    def print_magcal_usage(self):
+        print("Usage: magcal <start|accept|cancel|yaw>")
+
     def cmd_magcal(self, args):
         '''control magnetometer calibration'''
         if len(args) < 1:
-            print("Usage: magcal <start|accept|cancel>")
+            self.print_magcal_usage()
             return
 
         if args[0] == 'start':
@@ -156,7 +172,31 @@ class CalibrationModule(mp_module.MPModule):
                 0, # param5
                 0, # param6
                 0) # param7
-            
+        elif args[0] == 'yaw':
+            if len(args) < 2:
+                print("Usage: magcal yaw YAW_DEGREES <mask>")
+                return
+            yaw_deg = float(args[1])
+            mask = 0
+            if len(args) > 2:
+                mask = int(args[2])
+            print("Calibrating for yaw %.1f degrees with mask 0x%02x" % (yaw_deg, mask))
+            self.master.mav.command_long_send(
+                self.settings.target_system,  # target_system
+                0, # target_component
+                mavutil.mavlink.MAV_CMD_FIXED_MAG_CAL_YAW, # command
+                0, # confirmation
+                yaw_deg, # p1: yaw in degrees
+                mask, # p2: mask
+                0, # p3: lat_deg
+                0, # p4: lon_deg
+                0, # param5
+                0, # param6
+                0) # param7
+        else:
+            self.print_magcal_usage()
+            return
+
 def init(mpstate):
     '''initialise module'''
     return CalibrationModule(mpstate)
