@@ -29,9 +29,13 @@ from MAVProxy.modules.lib.graphdefinition import GraphDefinition
 from lxml import objectify
 import pkg_resources
 from builtins import input
+import datetime
+import matplotlib
 
 grui = []
 last_xlim = None
+xlim_low = None
+xlim_high = None
 flightmodes = None
 
 # Global var to hold the GUI menu element
@@ -54,6 +58,19 @@ class MEStatus(object):
     def __init__(self):
         self.msgs = {}
 
+def timestamp_in_range(timestamp):
+    '''check if a timestamp is in current xlim
+       return -1 if too low
+       return 1 if too high
+       return 0 if in range
+    '''
+    global xlim_low, xlim_high
+    if xlim_low is not None and timestamp < xlim_low:
+        return -1
+    if xlim_high is not None and timestamp > xlim_high:
+        return 1
+    return 0
+
 
 class MEState(object):
     '''holds state of MAVExplorer'''
@@ -201,10 +218,13 @@ def setup_menus():
     TopMenu.add(MPMenuSubMenu('Display',
                            items=[MPMenuItem('Map', 'Map', '# map'),
                                   MPMenuItem('Save Graph', 'Save', '# save'),
-                                  MPMenuItem('Reload Graphs', 'Reload', '# reload'),
-                                  MPMenuItem('FFT', 'FFT', '# fft')]))
+                                  MPMenuItem('Reload Graphs', 'Reload', '# reload')]))
     TopMenu.add(graph_menus())
     TopMenu.add(MPMenuSubMenu('FlightMode', items=flightmode_menu()))
+    TopMenu.add(MPMenuSubMenu('Tools',
+                              items=[MPMenuItem('MagFit', 'MagFit', '# magfit'),
+                                     MPMenuItem('Stats', 'Stats', '# stats'),
+                                     MPMenuItem('FFT', 'FFT', '# fft')]))
 
     mestate.console.set_menu(TopMenu, menu_callback)
 
@@ -404,9 +424,48 @@ def cmd_fft(args):
         condition = args[0]
     else:
         condition = None
-    child = multiproc.Process(target=mav_fft.mavfft_display, args=[mestate.filename,condition])
+    child = multiproc.Process(target=mav_fft.mavfft_display, args=[mestate.mlog,timestamp_in_range])
     child.start()
 
+def cmd_stats(args):
+    '''show status on log'''
+    from MAVProxy.modules.lib import msgstats
+    child = multiproc.Process(target=msgstats.show_stats, args=[mestate.mlog])
+    child.start()
+
+def cmd_dump(args):
+    '''dump messages from log'''
+    if len(args) > 0:
+        wildcard = args[0]
+    else:
+        print("Usage: dump PATTERN")
+        return
+    mlog = mestate.mlog
+    mlog.rewind()
+    types = []
+    for p in wildcard.split(','):
+        for t in mlog.name_to_id.keys():
+            if fnmatch.fnmatch(t, p):
+                types.extend([t])
+    while True:
+        msg = mlog.recv_match(type=types)
+        if msg is None:
+            break
+        in_range = timestamp_in_range(msg._timestamp)
+        if in_range < 0:
+            continue
+        if in_range > 0:
+            continue
+        print(msg)
+    mlog.rewind()
+
+def cmd_magfit(args):
+    '''fit magnetic field'''
+    from MAVProxy.modules.lib import magfit
+    mfit = magfit.MagFitUI(mestate.mlog, timestamp_in_range)
+    child = multiproc.Process(target=mfit.show)
+    child.start()
+    
 def save_graph(graphdef):
     '''save a graph as XML'''
     if graphdef.filename is None:
@@ -815,22 +874,23 @@ def main_loop():
             for c in cmds:
                 process_stdin(c)
 
+        remlist = []
         for i in range(0, len(grui)):
             xlim = grui[i].check_xlim_change()
             if xlim is not None and mestate.settings.sync_xzoom:
-                remlist = []
                 for j in range(0, len(grui)):
-                    #print("set_xlim: ", j, xlim)
+                    if j == i:
+                        continue
                     if not grui[j].set_xlim(xlim):
                         remlist.append(j)
                 last_xlim = xlim
-                if len(remlist) > 0:
-                    # remove stale graphs
-                    new_grui = []
-                    for j in range(0, len(grui)):
-                        if j not in remlist:
-                            new_grui.append(grui[j])
-                    grui = new_grui
+                from dateutil.tz import tzlocal
+                localtimezone = tzlocal()
+                tzofs = localtimezone.utcoffset(datetime.datetime.now()).total_seconds()
+                global xlim_low, xlim_high
+                xlim_low = matplotlib.dates.num2epoch(xlim[0]) - tzofs
+                xlim_high = matplotlib.dates.num2epoch(xlim[1]) - tzofs
+                
                 if mestate.settings.sync_xmap:
                     remlist = []
                     global map_timelim_pipes
@@ -840,6 +900,13 @@ def main_loop():
                         except Exception:
                             map_timelim_pipes.remove(p)
                 break
+        if len(remlist) > 0:
+            # remove stale graphs
+            new_grui = []
+            for j in range(0, len(grui)):
+                if j not in remlist:
+                    new_grui.append(grui[j])
+            grui = new_grui
 
         time.sleep(0.1)
 
@@ -857,6 +924,9 @@ command_map = {
     'map'        : (cmd_map,       'show map view'),
     'fft'        : (cmd_fft,       'show a FFT (if available)'),
     'loadLog'    : (cmd_loadfile,  'load a log file'),
+    'stats'      : (cmd_stats,     'show statistics on the log'),
+    'magfit'     : (cmd_magfit,    'fit mag parameters to WMM'),
+    'dump'       : (cmd_dump,      'dump messages from log'),
     }
 
 def progress_bar(pct):
